#include <assert.h>
#include <unistd.h> // for usleep()

#include <iostream>
#include <vector>
#include <string>
using namespace std;

#include <util/MediaItem.h>
#include <util/PluginOption.h>
#include <util/Playlist.h>
#include <core/IPlayer.h>
#include <core/IMediaLoader.h>
#include <core/IPluginManager.h>
#include <core/IConvTask.h>
#include <core/IConvTaskFactory.h>
using namespace mous;

#include <scx/Mutex.hpp>
#include <scx/Thread.hpp>
#include <scx/Signal.hpp>
using namespace scx;

bool gStop = false;
IPlayer* gPlayer = NULL;
Playlist<MediaItem>* gPlaylist = NULL;
Mutex gMutexForSwitch;

void OnFinished()
{
    gMutexForSwitch.Lock();
    if (gPlaylist != NULL && !gStop) {
        if (gPlaylist->SeqHasOffset(1)) {
            const MediaItem& item = gPlaylist->SeqItemAtOffset(1, true);
            if (gPlayer->Status() != PlayerStatus::Closed)
                gPlayer->Close();
            gPlayer->Open(item.url);
            if (item.hasRange)
                gPlayer->Play(item.msBeg, item.msEnd);
            else
                gPlayer->Play();
        }
    }
    gMutexForSwitch.Unlock();
    cout << "Finished!" << endl;
}

void OnPlaying()
{
    while (true) {
        if (gPlayer == NULL || gStop)
            break;
        gMutexForSwitch.Lock();
        uint64_t ms = gPlayer->OffsetMs();
        cout << gPlayer->BitRate() << " kbps " <<
            ms/1000/60 << ":" << ms/1000%60 << "." << ms%1000 << '\r' << flush;
        gMutexForSwitch.Unlock();
        usleep(200*1000);
    }
}

/*
#include <CharsetConv/CharsetConv.h>
//#include <enca.h>
*/

void PrintPluginOption(vector<PluginOption>& list)
{
    for (size_t i = 0; i < list.size(); ++i) {
        PluginOption& opt = list[i];
        cout << ">>>> index:" << i+1 << endl;
        cout << "\tplugin type: " << ToString(opt.pluginType)<< endl;
        for (size_t io = 0; io < opt.options.size(); ++io) {
            cout << "\t\t option type: " << ToString(opt.options[io]->type) << endl;
            cout << "\t\t option desc: " << opt.options[io]->desc << endl;
        }
    }
}

int main(int argc, char** argv)
{
    // Check args enough.
    if (argc < 2) {
        cout << "Usage:" << endl;
        cout << argv[0] << " [file1] [file2] [...]" << endl;
        return -1;
    }

    bool paused = false;

    /*
    string content(ReadAll(argv[1]));
    cout << "len:" << content.length() << endl;

    UErrorCode uerr = U_ZERO_ERROR;
    int32_t found = 1;
    UCharsetDetector* udec = ucsdet_open(&uerr);
    ucsdet_setText(udec, content.c_str(), content.length(), &uerr);
    const UCharsetMatch** match = ucsdet_detectAll(udec, &found, &uerr);
    for (int i = 0; i < found; ++i) {
        cout << ucsdet_getName(match[i], &uerr) << '\t';
        cout << ucsdet_getConfidence(match[i], &uerr) << endl;
    }
    cout << found << endl;
    ucsdet_close(udec);
    return 0;
    */

    /*
    CharsetConv conv;
    string output;
    bool ok = conv.AutoConv(content.c_str(), content.length(), output);
    cout << (ok ? output : content) << endl;
    return 0;
    */

    //EncaAnalyser ans = enca_analyser_alloc("uk");
    //EncaEncoding enc = enca_analyse_const(ans, (const unsigned char*)content.c_str(), content.length());
    //cout << enca_charset_name(enc.charset, ENCA_NAME_STYLE_ICONV) << endl;

    IPluginManager* mgr = IPluginManager::Create();
    mgr->LoadPluginDir("./plugins");

    // Dump all plugin path.
    vector<string> pathList;
    mgr->DumpPluginPath(pathList);
    for (size_t i = 0; i < pathList.size(); ++i) {
        cout << ">> " << pathList[i] << endl;
        const PluginInfo* info = mgr->QueryPluginInfo(pathList[i]);
        cout << ">>>> " << info->author << endl;
        cout << ">>>> " << info->name << endl;
        cout << ">>>> " << info->desc << endl;
        cout << ">>>> " << info->version << endl;
    }
    cout << endl;

    // Get all plugin agents.
    vector<const IPluginAgent*> decoderAgentList;
    mgr->DumpPluginAgent(decoderAgentList, PluginType::Decoder);
    cout << ">> Decoder count:" << decoderAgentList.size() << endl;

    vector<const IPluginAgent*> encoderAgentList;
    mgr->DumpPluginAgent(encoderAgentList, PluginType::Encoder);
    cout << ">> Encoder count:" << encoderAgentList.size() << endl;

    vector<const IPluginAgent*> rendererAgentList;
    mgr->DumpPluginAgent(rendererAgentList, PluginType::Renderer);
    cout << ">> Renderer count:" << rendererAgentList.size() << endl;

    vector<const IPluginAgent*> packAgentList;
    mgr->DumpPluginAgent(packAgentList, PluginType::MediaPack);
    cout << ">> MediaPack count:" << packAgentList.size() << endl;

    vector<const IPluginAgent*> tagAgentList;
    mgr->DumpPluginAgent(tagAgentList, PluginType::TagParser);
    cout << ">> TagParser count:" << tagAgentList.size() << endl;
    cout << endl;

    vector<const IPluginAgent*> pelAgentList;
    mgr->DumpPluginAgent(pelAgentList, PluginType::EventWatcher);
    cout << ">> EventWatcher count:" << pelAgentList.size() << endl;

    // Check plugins enough.
    if (decoderAgentList.empty() || rendererAgentList.empty())
        return -2;

    // Setup loader
    IMediaLoader* loader = IMediaLoader::Create();
    loader->RegisterMediaPackPlugin(packAgentList);
    loader->RegisterTagParserPlugin(tagAgentList);

    // Setup playlist
    Playlist<MediaItem> playlist;
    gPlaylist = &playlist;
    deque<MediaItem> mediaList;
    for (int i = 1; i < argc; ++i) {
        loader->LoadMedia(argv[i], mediaList);
        playlist.Append(mediaList);
    }
    playlist.SetMode(PlaylistMode::Repeat);

    // test for encoder
    if (false)
    {
        IConvTaskFactory* factory = IConvTaskFactory::Create();
        factory->RegisterDecoderPlugin(encoderAgentList);
        factory->RegisterEncoderPlugin(decoderAgentList);

        vector<string> encoders = factory->EncoderNames();
        if (encoders.empty()) {
            cout << "No encoders!" << endl;
            IConvTaskFactory::Free(factory);
            return 0;
        }

        cout << ">> Available encoders:" << endl;
        for (size_t i = 0; i < encoders.size(); ++i) {
            cout << i+1 << ": " << encoders[i] << endl;
        }
        int index;
        cout << ">> Select encoder:";
        cin >> index;
        if (index < 1) {
            IConvTaskFactory::Free(factory);
            return 0;
        }

        MediaItem item = playlist[11];
        cout << item.url << endl;
        IConvTask* task = factory->CreateTask(item, encoders[index-1]);
        task->Run("output.wav");

        while (!task->IsFinished()) {
            double percent =  task->Progress();
            if (percent < 0) {
                cout << "failed!" << endl;
                break;
            }
            cout << "progress:" << (int)(percent*100) << "%" << "\r" << flush;
            usleep(1000);
        }
        cout << "quit!" << endl;

        IConvTask::Free(task);
        IConvTaskFactory::Free(factory);
        return 0;
    }

    // Setup player
    IPlayer* player = IPlayer::Create();
    player->SigFinished()->Connect(&OnFinished);
    player->RegisterRendererPlugin(rendererAgentList[0]);
    player->RegisterDecoderPlugin(decoderAgentList);
    for (size_t i = 0; i < pelAgentList.size(); ++i) {
        //player->RegisterPlugin(pelAgentList[i]);
    }

    // Show player options 
    {
        vector<PluginOption> list;
        cout << ">> Player decoder plugin options:" << endl;
        player->DecoderPluginOption(list);
        PrintPluginOption(list);
        cout << ">> Player renderer plugin options:" << endl;
        PluginOption opt;
        player->RendererPluginOption(opt);
        list.resize(1);
        list[0] = opt;
        PrintPluginOption(list);
    }

    // Begin to play.
    if (playlist.Empty())
        return -1;

    const MediaItem& item = playlist.SeqItemAtOffset(0, false);

    cout << ">>>> Tag Info" << endl;
    cout << "\ttitle(" << item.tag.title.size() << "):" << item.tag.title << endl;
    cout << "\tartist(" << item.tag.artist.size() << "):" << item.tag.artist << endl;
    cout << "\talbum(" << item.tag.album.size() << "):" << item.tag.album << endl;
    cout << "\tcomment:" << item.tag.comment << endl;
    cout << "\tgenre:" << item.tag.genre << endl;
    cout << "\tyear:" << item.tag.year << endl;
    cout << "\ttrack:" << item.tag.track << endl;

    cout << "item.url:" << item.url << endl;
    player->Open(item.url);
    if (item.hasRange) {
        player->Play(item.msBeg, item.msEnd);
    } else {
        player->Play();
    }
    Thread th;
    gPlayer = player;
    th.Run(Function<void (void)>(&OnPlaying));

    char ch = ' ';
    while (ch != 'q') {
        cin >> ch;
        switch (ch) {
            case 'q':
                player->Close();
                break;

            case 'p':
                if (paused) {
                    player->Resume();
                    paused = false;
                } else {
                    player->Pause();
                    paused = true;
                }
                break;

            case 'r':
                if (item.hasRange) {
                    player->Play(item.msBeg, item.msEnd);
                } else {
                    player->Play();
                }
                break;
        }
    }

    gStop = true;
    th.Join();

    loader->UnregisterAll();
    player->UnregisterAll();
    mgr->UnloadAll();
    
    IPlayer::Free(player);
    IMediaLoader::Free(loader);
    IPluginManager::Free(mgr);

    return 0;
}
